import React, { useCallback, useState, useMemo } from 'react';
import {
  scaleTime,
  extent,
  scaleLinear,
  max,
  line,
  timeFormat,
  scaleLog,
} from 'd3';
import { VoronoiOverlay } from './VoronoiOverlay';

import { XAxis } from './XAxis';
import { YAxis } from './YAxis';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  top: 40,
  right: 20,
  bottom: 60,
  left: 80,
};

const formatDate = timeFormat('%b %d');

export const LineChart = ({ data, width, height }) => {
  const [activeCountryName, setActiveCountryName] = useState();
  
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const allData = useMemo(() => data.reduce(
    (accumulator, countryTimeseries) =>
      accumulator.concat(countryTimeseries),
    []
  ), [data]);

  const epsilon = 1;

  const xScale = useMemo(() => scaleTime()
    .domain(extent(allData, xValue))
    .range([0, innerWidth]), [allData, xValue]);

  const yScale = useMemo(() => scaleLog()
    .domain([epsilon, max(allData, yValue)])
    .range([innerHeight, 0]), [allData, yValue]);

  const lineGenerator = useMemo(() => line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(epsilon + yValue(d))), 
  [xScale, xValue, yScale, yValue, epsilon]);

  const mostRecentDate = xScale.domain()[1];

  console.log(activeCountryName);
  
  const handleVoronoiHover = useCallback(d => {
   // console.log('Hovered');
  //  console.log(d)
    setActiveCountryName(d.countryName);
  }, []);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((countryTimeseries) => {
          return (
            <path
              className="marker-line"
              d={lineGenerator(countryTimeseries)}
            />
          );
        })}
        { activeCountryName ? <path
          className="marker-line active"
          d={lineGenerator(data.find(countryTimeseries => countryTimeseries.countryName === activeCountryName))}
        />: null}
        <text>Global Coronavirus Deaths Over Time</text>
        <text
          className="axis-label"
          transform={`translate(-45, ${
            innerHeight / 2
          }) rotate(-90)`}
          text-anchor="middle"
        >
          Cumulative Deaths
        </text>
        <text
          className="axis-label"
          transform={`translate(${innerWidth / 2}, ${
            innerHeight + 30
          })`}
          alignment-baseline="hanging"
          text-anchor="middle"
        >
          Time
        </text>
        <VoronoiOverlay
          onHover={handleVoronoiHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
        />
      </g>
    </svg>
  );
};
